% The following code is a compression Demo for images and for a graph 
% signal on the Minnesota traffic graph (uncomment necessary lines).
%
% This file is part of the CoSub toolbox (Connected Subgraph Filterbanks 
% For Graph Signals Toolbox)
% Copyright (C) 2015, Nicolas Tremblay. 
%
% The CoSub toolbox is free software: you can redistribute 
% it and/or modify it under the terms of the GNU General 
% Public License as published by the Free Software 
% Foundation, either version 3 of the License, or (at your 
% option) any later version.
%
% The CoSub toolbox is distributed in the hope 
% that it will be useful, but WITHOUT ANY WARRANTY; 
% without even the implied warranty of MERCHANTABILITY 
% or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
% Public License for more details.
%
% You should have received a copy of the GNU General Public 
% License along with the MSCD_Wav toolbox. 
% If not, see <http://www.gnu.org/licenses/>.

clear all; close all; clc;

global GLOBAL_CSFB; global strategy_CSFB; global norm_Q_CSFB; global MAX_COM_CSFB; 
CSFB_start;

%% =====================================================================
%% ============================= choose data ===========================
%% =====================================================================

%% -------------------- image ------------------------------------------
% choose an image in 'Datasets' folder
im=1;
% filename='CSFB_sample1.png';
% filename='CSFB_coins.png';
filename='CSFB_cameraman.tif';

%f=rgb2gray(imread([GLOBAL_CSFB,filesep,'Datasets',filesep,filename]));
f=imread([GLOBAL_CSFB,filesep,'Datasets',filesep,filename]);
sizeim=size(f); f=double(f(:)); N=length(f);
A=CSFB_grid_graph2d(sizeim(1),sizeim(2));

%% ------------------- Minnesota traffic graph -------------------------
% % A is the structure on which the graph signal f is defined.
% im=0;
% load([GLOBAL_CSFB,filesep,'Datasets',filesep,'CSFB_min_traffic_graph.mat']);
% A = double(A>0);
% load([GLOBAL_CSFB,filesep,'Datasets',filesep,'CSFB_min_graph_signal.mat']);
% locationsA=xy;
% % remove two nodes (disconnected):
% A(348:349,:)=[];A(:,348:349)=[];
% locationsA(348:349,:)=[];
% f(348:349)=[];
% N=length(f); A=sparse(A);
% 
% % if you want to analyse a "smooth" signal instead of the default
% % piece-wise constant signal:
% L=CSFB_create_laplacian(A,'combinatorial');
% [U,D]=eig(full(L)); U=U(:,1:5);
% if U(1,1)<0
%     U=-U;end; f=sum(U,2); f=f./max(abs(f));
% 
% % if you want to analyse a random signal of same eneregy than the previous
% % "smooth" signal, add:
% norm_fs=norm(f);
% f=randn(size(f)); f=f./norm(f); f=f.*norm_fs;

%% =====================================================================
%% ========================= choose parameters =========================
%% =====================================================================

% choose one of the following clustering methods
% clustering_method='regular_2d_grid'; %2*2 blocks of pixel
% clustering_method='Louvain'; %Louvain on adjacency matrix
clustering_method='Ed_Aw_Louvain'; %Louvain on edge-aware adjacency matrix
% clustering_method='Infomap'; %Infomap on adjacency matrix
% clustering_method='Ed_Aw_Infomap'; %Infomap on edge-aware adjacency matrix

% choose strategy
% strategy_CSFB='largest'; % subgraphs as large as possible (but still max(|subgraph|) < MAX_COM_CSFB nodes)
MAX_COM_CSFB=1000; % maximum number of nodes in any subgraph (useful only if strategy_CSFB='largest')
strategy_CSFB='smallest'; % subgraphs as small as possible (clustering algo stops after first pass)

% choose normalisation
% norm_Q_CSFB=2; % L2 normalisation
norm_Q_CSFB=1; % L1 normalisation

% choose depth of cascade
% note that depth=2 means a one-level cascade
depth=4;

% Reconstruction is done with all low pass coefficients and some
% percentage of high-pass coefficients. Please enter here the desired
% overall compression rate, i.e.
% compr_ratio = (Uncompressed data size)/(Compressed data size)
compr_ratio=20; %must be superior to 1 

%% =====================================================================
%% ==========================  CSFB cascade ============================
%% =====================================================================

fprintf('Computing forward-cascade of CoSub Filterbanks...');
[f0,Ai,COM,num_LP]=CSFB_compute_all_levels(f,A,clustering_method,depth);
% f0 is a cell-type with 'depth' cells:
% - Cell #1 contains the original signal (of size N)
% - Cell #i contains a vector of size N. The num_LP(i) first coefficients are the
% approximation coefficients at this level. The subsequent coefficients are
% detail coefficients.
% Ai is a cell-type with 'depth - 1' cells:
% - Cell #i contains the "interior Adjacency matrix", i.e. the concatenated
% Adjacency matrices of all subgraphs
% COM is a cell-type with 'depth' cells:
% - Cell 1 is empty.
% - Cell i is the subgraph labeling of the graph on which the approximated
% signal at level (i-1) is defined.

fprintf('\t\t\t\t Done.\n\n');

if length(f0)~=depth
    depth=length(f0);
    warning(['The depth you chose is too large: the cascade stopped at max depth=', num2str(depth)])
end

%% =====================================================================
%% ========================== Reconstruction ===========================
%% =====================================================================

perc_HF=(N/compr_ratio-num_LP(depth))/(N-num_LP(depth));
% that's the percentage of kept High Pass coefficients.
if perc_HF<0
    % The desired compression rate is not possible with such small number of cascade levels!
    error('The desired compression rate is not possible with such small number of cascade levels! Either increase the depth of the cascade or decrease the compression ratio');
end

fprintf('Reconstructing non-linear approximation (backward-cascade)...');
frec=CSFB_reconstruct(f0{depth},Ai(1:depth-1),perc_HF,num_LP(depth));
% Reconstruction needs:
% - f0{depth}: the transformed signal at level 'depth'
% - Ai(1:depth-1): all the "interior Adjacency matrices" up to level 'depth - 1'
% - perc_HF: the percentage of kept HP coef.
% - num_LP(depth): that is just to indicate where are the LP coef of
% f0{depth}.
fprintf('\t\t\t Done.\n\n');

%% =====================================================================
%% =========================== Show Results ============================
%% =====================================================================

if im==1
    % compute measures of quality
    rec_q=CSFB_compute_image_quality(f,frec);
    
    % and plot
    fprintf('Plotting results...');
    f1=figure(1); a1 = axes('parent',f1); imshow(reshape(uint8(f),sizeim(1),sizeim(2)));
    title(a1,['original image w/ ' num2str(N) ' total coef.']);
    set(f1,'position',[281   568   446   335]);
    
    f2=figure(2); a2 = axes('parent',f2); imshow(reshape(uint8(frec),sizeim(1),sizeim(2)));
    title(a2,['rec. im. w/ #LP coef=' num2str(num_LP(depth)) '; #HP coef=' num2str(round(perc_HF*(N-num_LP(depth)))) '; PSNR=' num2str(round(rec_q.PSNR*100)/100)]);
    set(f2,'position',[727   568   456   328]);
    
    f3=figure(3); a3 = axes('parent',f3); imshow(reshape(uint8(abs(f-frec)),sizeim(1),sizeim(2)));
    title(a3,'reconstruction error');
    set(f3,'position',[1183  568  466   321]);
    
    % plot quick stats about the subgraph sizes of the first level
    % decomposition:
    sizeCOM2=zeros(1,max(COM{2}));
    for i=1:max(COM{2})
        sizeCOM2(i)=length(find(COM{2}==i));
    end
    f4=figure; a4 = axes('parent',f4); hist(sizeCOM2,100);set(f4,'position',[290   136   436   342]);
    title(a4,'histogram of subgraph size (1st level)');
    
    [m,ind]=max(sizeCOM2);f5=figure; a5 = axes('parent',f5); imshow(reshape(COM{2}==ind,sizeim(1),sizeim(2))); set(f5,'position',[ 726   141   446   335]);
    title(a5,'largest subgraph (1st level)');
    
    [m,ind]=min(sizeCOM2);f6=figure; a6 = axes('parent',f6); imshow(reshape(COM{2}==ind,sizeim(1),sizeim(2))); set(f6,'position',[1172  148  456 328]);
    title(a6,'smallest subgraph (1st level)');
    
    fprintf('\t\t\t\t\t\t\t\t Done.\n\n');
    
    fprintf('Image reconstruction quality:\n')
    display(rec_q);
else
    % compute measures of quality
    rec_q=CSFB_compute_image_quality(f,frec);
    
    % and plot
    fprintf('Plotting results...');
    
    CLim=[min([f;frec]),max([f;frec])];
    f1=figure;  a1 = axes('parent',f1); CSFB_ggplot(A,locationsA,10,0,1,f,CLim);box off; axis off;
    title(a1,['original signal w/ ' num2str(N) ' total coef.']);
    set(f1,'position',[90   529   560   420]);
    
    f2=figure;  a2 = axes('parent',f2); CSFB_ggplot(A,locationsA,10,0,1,frec,CLim);box off; axis off;
    title(a2,['rec. signal w/ #LP coef=' num2str(num_LP(depth)) '; #HP coef=' num2str(round(perc_HF*(N-num_LP(depth)))) ';   PSNR=' num2str(round(rec_q.PSNR*100)/100)]);
    set(f2,'position',[652   531   560   420]);
    
    f3=figure;  a3 = axes('parent',f3); CSFB_ggplot(A,locationsA,10,0,1,abs(f-frec),CLim);box off; axis off;
    title(a3,'reconstruction error');
    set(f3,'position',[1212 531 560 420]);
    
    % plot quick stats about the subgraph sizes of the first level
    % decomposition:
    sizeCOM=zeros(1,max(COM{2}));
    for i=1:max(COM{2})
        sizeCOM(i)=length(find(COM{2}==i));
    end
    f4=figure;a4 = axes('parent',f4); hist(sizeCOM,100);title(a4,'histogram of subgraph size (1st level)');
    set(f4,'position',[714    97   436   342]);
    
    fprintf('\t\t\t\t\t\t\t\t Done.\n\n');
    
    fprintf('Signal reconstruction quality:\n')
    display(rec_q);
end
