% The following code is a denoising Demo for different graph signals 
% on the Minnesota traffic graph.
%
% This file is part of the CoSub toolbox (Connected Subgraph Filterbanks 
% For Graph Signals Toolbox)
% Copyright (C) 2015, Nicolas Tremblay. 
%
% The CoSub toolbox is free software: you can redistribute 
% it and/or modify it under the terms of the GNU General 
% Public License as published by the Free Software 
% Foundation, either version 3 of the License, or (at your 
% option) any later version.
%
% The CoSub toolbox is distributed in the hope 
% that it will be useful, but WITHOUT ANY WARRANTY; 
% without even the implied warranty of MERCHANTABILITY 
% or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
% Public License for more details.
%
% You should have received a copy of the GNU General Public 
% License along with the MSCD_Wav toolbox. 
% If not, see <http://www.gnu.org/licenses/>.

clear all; close all; clc; 

global GLOBAL_CSFB; global strategy_CSFB; global norm_Q_CSFB; global MAX_COM_CSFB; 
CSFB_start;

%% =====================================================================
%% ============================= choose data ===========================
%% =====================================================================

%% ------------------- Minnesota traffic graph -------------------------
% A is the structure on which the graph signal f is defined.
im=0;
load([GLOBAL_CSFB,filesep,'Datasets',filesep,'CSFB_min_traffic_graph.mat']);
A = double(A>0);
load([GLOBAL_CSFB,filesep,'Datasets',filesep,'CSFB_min_graph_signal.mat']);
locationsA=xy;
% remove two nodes (disconnected):
A(348:349,:)=[];A(:,348:349)=[];
locationsA(348:349,:)=[];
f(348:349)=[];
N=length(f); A=sparse(A);

% if you want to analyse a "smooth" signal instead of the default
% piece-wise constant signal:
L=CSFB_create_laplacian(A,'combinatorial');
[U,D]=eig(full(L)); U=U(:,1:5);
if U(1,1)<0
U=-U;end; f=sum(U,2); f=f./max(abs(f));

%% =====================================================================
%% ========================= choose parameters =========================
%% =====================================================================

% choose one of the following clustering methods
% clustering_method='regular_2d_grid'; %2*2 blocks of pixel
% clustering_method='Louvain'; %Louvain on adjacency matrix
clustering_method='Ed_Aw_Louvain'; %Louvain on edge-aware adjacency matrix
% clustering_method='Infomap'; %Infomap on adjacency matrix
% clustering_method='Ed_Aw_Infomap'; %Infomap on edge-aware adjacency matrix

% choose strategy
strategy_CSFB='largest'; % subgraphs as large as possible (but still max(|subgraph|) < MAX_COM_CSFB nodes)
MAX_COM_CSFB=1000; % maximum number of nodes in any subgraph (useful only if strategy_CSFB='largest')
% strategy_CSFB='smallest'; % subgraphs as small as possible (clustering algo stops after first pass)

% choose normalisation
norm_Q_CSFB=2; % L2 normalisation
% norm_Q_CSFB=1; % L1 normalisation

% choose the level of corruption of the graph signal
sigm=0.25; %std of additive Gaussian noise

%% corrupt the graph signal with additive Gaussian noise
f_noisy=f+sigm.*randn(N,1);

%% =====================================================================
%% ========================== One level of CSFB ========================
%% =====================================================================

% depth of cascade (should be at depth=2 for denoising)
% note that depth=2 means a one-level cascade
depth=2;

fprintf('Computing forward-cascade of CoSub Filterbanks...');
[f0,Ai,COM,num_LP]=CSFB_compute_all_levels(f_noisy,A,clustering_method,depth);
% f0 is a cell-type with 'depth' cells:
% - Cell #1 contains the original signal (of size N)
% - Cell #i contains a vector of size N. The num_LP(i) first coefficients are the
% approximation coefficients at this level. The subsequent coefficients are
% detail coefficients.
% Ai is a cell-type with 'depth - 1' cells:
% - Cell #i contains the "interior Adjacency matrix", i.e. the concatenated
% Adjacency matrices of all subgraphs
% COM is a cell-type with 'depth' cells:
% - Cell 1 is empty.
% - Cell i is the subgraph labeling of the graph on which the approximated
% signal at level (i-1) is defined.
fprintf('\t\t\t\t Done.\n\n');

%% =====================================================================
%% ========== Reconstruction after hard-thresholding ===================
%% =====================================================================

f0=f0{depth}; details=f0(num_LP(depth)+1:end);

details(abs(details)<3*sigm)=0; % hard-thresholding at 3sigma
f_th=f0; f_th(num_LP(depth)+1:end)=details;

fprintf('Reconstructing denoised signal (backward-cascade)...');
frec=CSFB_compute_all_levels_backwards(f_th,Ai(1:depth-1));
% Reconstruction needs:
% - f_th: thresholded signal decomposition
% - Ai(1:depth-1): all the "interior Adjacency matrices" up to 'depth - 1'
fprintf('\t\t\t\t Done.\n\n');

%% plot results

fprintf('Plotting results...');
CLim=[min([f_noisy;f;frec]),max([f_noisy;f;frec])];

f1=figure; a1 = axes('parent',f1); CSFB_ggplot(A,locationsA,10,0,0,f,CLim);box off; axis off;
title(a1,'f original');
set(f1,'position',[90   529   560   420]);

f2=figure; a2 = axes('parent',f2);CSFB_ggplot(A,locationsA,10,0,0,f_noisy,CLim);box off; axis off;
title(a2,['noisy f (MSE=' num2str(mse(f,f_noisy),3) ', SNR=' num2str(snr(f,f_noisy-f),3) ')']);
set(f2,'position',[652   557   560   420]);

f3=figure; a3 = axes('parent',f3);CSFB_ggplot(A,locationsA,10,0,0,frec,CLim);box off; axis off;
title(a3,['f denoised (MSE=' num2str(mse(f,frec),3) ', SNR=' num2str(snr(f,frec-f),3) ')']);
set(f3,'position',[1218  557   560  420]);
fprintf('\t\t\t\t\t\t\t\t Done.\n\n');

