% The following code plots successive approximation graph signals of a
% smooth graph signal on the Minnesota traffic graph.
%
% This file is part of the CoSub toolbox (Connected Subgraph Filterbanks 
% For Graph Signals Toolbox)
% Copyright (C) 2015, Nicolas Tremblay. 
%
% The CoSub toolbox is free software: you can redistribute 
% it and/or modify it under the terms of the GNU General 
% Public License as published by the Free Software 
% Foundation, either version 3 of the License, or (at your 
% option) any later version.
%
% The CoSub toolbox is distributed in the hope 
% that it will be useful, but WITHOUT ANY WARRANTY; 
% without even the implied warranty of MERCHANTABILITY 
% or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
% Public License for more details.
%
% You should have received a copy of the GNU General Public 
% License along with the MSCD_Wav toolbox. 
% If not, see <http://www.gnu.org/licenses/>.


clear all; close all; clc;

global GLOBAL_CSFB; global strategy_CSFB; global norm_Q_CSFB; global MAX_COM_CSFB; 
CSFB_start;

%% =====================================================================
%% ========================= choose parameters =========================
%% =====================================================================

% choose one of the following clustering methods
% clustering_method='regular_2d_grid'; %2*2 blocks of pixel
clustering_method='Louvain'; %Louvain on adjacency matrix
% clustering_method='Ed_Aw_Louvain'; %Louvain on edge-aware adjacency matrix
% clustering_method='Infomap'; %Infomap on adjacency matrix
% clustering_method='Ed_Aw_Infomap'; %Infomap on edge-aware adjacency matrix

% choose strategy
% strategy_CSFB='largest'; % subgraphs as large as possible (but still max(|subgraph|) < MAX_COM_CSFB nodes)
MAX_COM_CSFB=1000; % maximum number of nodes in any subgraph (useful only if strategy_CSFB='largest')
strategy_CSFB='smallest'; % subgraphs as small as possible (clustering algo stops after first pass)

% choose normalisation
% norm_Q_CSFB=2; % L2 normalisation
norm_Q_CSFB=1; % L1 normalisation

% choose depth of cascade
% note that depth=2 means a one-level cascade
depth=7;

%% =====================================================================
%% ========================= load Minnesota graph ======================
%% =====================================================================

% A is the structure on which the graph signal f is defined.
load([GLOBAL_CSFB,filesep,'Datasets',filesep,'CSFB_min_traffic_graph.mat']);
A = double(A>0);
locationsA=xy;
% remove two nodes (disconnected) :
A(348:349,:)=[];A(:,348:349)=[];
locationsA(348:349,:)=[];
A=sparse(A);

load([GLOBAL_CSFB,filesep,'Datasets',filesep,'CSFB_min_graph_signal.mat']);
f(348:349)=[];
% if you want to analyse a "smooth" signal instead of the default
% piece-wise constant signal:
L=CSFB_create_laplacian(A,'combinatorial');
[U,D]=eig(full(L)); U=U(:,1:5);
if U(1,1)<0
    U=-U;end; f=sum(U,2); f=f./max(abs(f));

%% =====================================================================
%% ==========================  CSFB cascade ============================
%% =====================================================================

fprintf('Computing forward-cascade of CoSub Filterbanks...');
[f0,Ai,COM,num_LP]=CSFB_compute_all_levels(f,A,clustering_method,depth);
% f0 is a cell-type with 'depth' cells:
% - Cell #1 contains the original signal (of size N)
% - Cell #i contains a vector of size N. The num_LP(i) first coefficients are the
% approximation coefficients at this level. The subsequent coefficients are
% detail coefficients.
% Ai is a cell-type with 'depth - 1' cells:
% - Cell #i contains the "interior Adjacency matrix", i.e. the concatenated
% Adjacency matrices of all subgraphs
% COM is a cell-type with 'depth' cells:
% - Cell 1 is empty.
% - Cell i is the subgraph labeling of the graph on which the approximated
% signal at level (i-1) is defined.
fprintf('\t\t\t\t Done.\n\n');

if length(f0)~=depth
    depth=length(f0);
    warning(['The depth you chose is too large: the cascade stopped at max depth=', num2str(depth)])
end

%% =====================================================================
%% ===================== Get all partial recoveries ====================
%% =====================================================================

for max_level=1:depth-1
    frec{max_level}=CSFB_reconstruct(f0{max_level},Ai(1:max_level-1),0,num_LP(max_level));
end

fprintf('Plotting results...');
mini=max(f0{1}); maxi=min(f0{1});
for i=1:depth-1
    mini=min([mini;f0{i}(1:num_LP(i))]);
    mini=min([mini;frec{i}]);
    maxi=max([maxi;f0{i}(1:num_LP(i))]);
    maxi=max([maxi;frec{i}]);
end
CLim=[mini,maxi];

figure('units','normalized','outerposition',[0 0 1 1])
subplot(3,4,1);
CSFB_ggplot(A,locationsA,10,0,0,f0{1},CLim);box off; axis off;
title('original')

% first scale
[Nk,~] = histc(COM{2},unique(COM{2}));
Ntilde=max(Nk);
[Ai,Ae]=CSFB_separate_Adjacency(A,COM{2});
[S,Q]=CSFB_get_Q_ordered(Ai,COM{2},Nk,Ntilde,'next');
A0=S{1}'*Ae*S{1};

locationsA0=zeros(max(COM{2}),2);
for i=1:max(COM{2})
    locationsA0(i,:)=mean(locationsA(COM{2}==i,:));
end

subplot(3,4,5);CSFB_ggplot(A0,locationsA0,10,0,0,f0{2}(1:num_LP(2)),CLim);box off; axis off;
title('1st scale approx')
subplot(3,4,6); CSFB_ggplot(A,locationsA,10,0,0,frec{2},CLim);box off; axis off;
title('1st scale recovery')

% second scale
[Nk,~] = histc(COM{3},unique(COM{3}));
Ntilde=max(Nk);
[Ai,Ae]=CSFB_separate_Adjacency(A0,COM{3});
[S,Q]=CSFB_get_Q_ordered(Ai,COM{3},Nk,Ntilde,'next');
A1=S{1}'*Ae*S{1};

locationsA1=zeros(max(COM{3}),2);
for i=1:max(COM{3})
    locationsA1(i,:)=mean(locationsA0(COM{3}==i,:));
end

subplot(3,4,9); CSFB_ggplot(A1,locationsA1,10,0,0,f0{3}(1:num_LP(3)),CLim);box off; axis off;
title('2nd scale approx')
subplot(3,4,10); CSFB_ggplot(A,locationsA,10,0,0,frec{3},CLim);box off; axis off;
title('2nd scale recovery')

% third scale
[Nk,~] = histc(COM{4},unique(COM{4}));
Ntilde=max(Nk);
[Ai,Ae]=CSFB_separate_Adjacency(A1,COM{4});
[S,Q]=CSFB_get_Q_ordered(Ai,COM{4},Nk,Ntilde,'next');
A2=S{1}'*Ae*S{1};

locationsA2=zeros(max(COM{4}),2);
for i=1:max(COM{4})
    locationsA2(i,:)=mean(locationsA1(COM{4}==i,:));
end

subplot(3,4,3); CSFB_ggplot(A2,locationsA2,10,0,0,f0{4}(1:num_LP(4)),CLim);box off; axis off;
title('3rd scale approx')
subplot(3,4,4); CSFB_ggplot(A,locationsA,10,0,0,frec{4},CLim);box off; axis off;
title('3rd scale recovery')

% fourth scale
[Nk,~] = histc(COM{5},unique(COM{5}));
Ntilde=max(Nk);
[Ai,Ae]=CSFB_separate_Adjacency(A2,COM{5});
[S,Q]=CSFB_get_Q_ordered(Ai,COM{5},Nk,Ntilde,'next');
A3=S{1}'*Ae*S{1};

locationsA3=zeros(max(COM{5}),2);
for i=1:max(COM{5})
    locationsA3(i,:)=mean(locationsA2(COM{5}==i,:));
end

subplot(3,4,7); CSFB_ggplot(A3,locationsA3,10,0,0,f0{5}(1:num_LP(5)),CLim);box off; axis off;
title('4th scale approx')
subplot(3,4,8); CSFB_ggplot(A,locationsA,10,0,0,frec{5},CLim);box off; axis off;
title('4th scale recovery')

% fifth scale
[Nk,~] = histc(COM{6},unique(COM{6}));
Ntilde=max(Nk);
[Ai,Ae]=CSFB_separate_Adjacency(A3,COM{6});
[S,Q]=CSFB_get_Q_ordered(Ai,COM{6},Nk,Ntilde,'next');
A4=S{1}'*Ae*S{1};

locationsA4=zeros(max(COM{6}),2);
for i=1:max(COM{6})
    locationsA4(i,:)=mean(locationsA3(COM{6}==i,:));
end

subplot(3,4,11); CSFB_ggplot(A4,locationsA4,10,0,0,f0{6}(1:num_LP(6)),CLim);box off; axis off;
title('5th scale approx')
subplot(3,4,12); CSFB_ggplot(A,locationsA,10,0,0,frec{6},CLim);box off; axis off;
title('5th scale recovery')

fprintf('\t\t\t\t\t\t\t\t Done.\n\n');
