% The following code is a compression demo using graph biorthogonal
% filterbanks
% 
% This file is part of the CoSub toolbox (Connected Subgraph Filterbanks 
% For Graph Signals Toolbox)
% Copyright (C) 2015, Nicolas Tremblay. 
%
% The CoSub toolbox is free software: you can redistribute 
% it and/or modify it under the terms of the GNU General 
% Public License as published by the Free Software 
% Foundation, either version 3 of the License, or (at your 
% option) any later version.
%
% The CoSub toolbox is distributed in the hope 
% that it will be useful, but WITHOUT ANY WARRANTY; 
% without even the implied warranty of MERCHANTABILITY 
% or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
% Public License for more details.
%
% You should have received a copy of the GNU General Public 
% License along with the MSCD_Wav toolbox. 
% If not, see <http://www.gnu.org/licenses/>.

clear all; close all; clc;

global GLOBAL_CSFB;
CSFB_start;

%% =====================================================================
%% ============================= choose data ===========================
%% =====================================================================

% choose an image in 'Datasets' folder
% The image should be square for this GrBior code to work

filename='CSFB_cameraman.tif';
% filename='CSFB_samplesquare.png';
% filename='CSFB_coinssquare.png';

f=imread([GLOBAL_CSFB,filesep,'Datasets',filesep,filename]);
sizeim=size(f); f=double(f(:)); N=length(f);

%% =====================================================================
%% ========================= choose parameters =========================
%% =====================================================================

vanishing_moments=[6,6];
norm_type='asym'; % coulb be "sym" if one wants to use the symmetrical normalized Laplacian
% results are usually better wth "asym"
edgemap=1; % =1 if edge-awareness. =0 if not

% choose depth of cascade
% note that depth=1 means a one-level cascade
depth=3;

% Reconstruction is done with all low pass coefficients and some
% percentage of high-pass coefficients. Please enter here the desired
% overall compression rate, i.e.
% compr_ratio = (Uncompressed data size)/(Compressed data size)
compr_ratio=20; %must be superior to 1

%% =====================================================================
%% ================ decompose, threshold and recover ===================
%% =====================================================================

if sizeim(1)~=sizeim(2)
    error('Image should be square to work with this code. In fact, bipartite decomposition is NOT computed here..');
end
LPcoef=sizeim(1)^2/4^depth;
if round(LPcoef)~=LPcoef
    error('The desired depth is not possible: either increase size of image or decrease depth');
end

perc_HF=(sizeim(1)^2/compr_ratio-LPcoef)/(sizeim(1)^2-LPcoef);
% that's the percentage of kept High Pass coefficients.
if perc_HF<0
    % The desired compression rate is not possible with such small number of cascade levels!
    error('The desired compression rate is not possible with such small number of cascade levels! Either increase the depth of the cascade or decrease the compression ratio');
end

fprintf('Decompose and recover with Graph Bior...');
[Data,frec,num_LP] = NLA_GrBior(filename,depth,perc_HF,edgemap,vanishing_moments,norm_type);
fprintf('\t\t\t\t\t Done.\n\n');

%% =====================================================================
%% =========================== plot results ============================
%% =====================================================================

% compute measures of quality
rec_q=CSFB_compute_image_quality(f,frec);

% and plot
fprintf('Plotting results...');
f1=figure(1); a1 = axes('parent',f1); imshow(reshape(uint8(f),sizeim(1),sizeim(2)));
title(a1,['original image w/ ' num2str(N) ' total coef.']);
set(f1,'position',[281   568   446   335]);

f2=figure(2); a2 = axes('parent',f2); imshow(reshape(uint8(frec),sizeim(1),sizeim(2)));
title(a2,['rec. im. #LP coef=' num2str(num_LP(2)) '; #HP coef=' num2str(round(perc_HF*(N-num_LP(2)))) '; PSNR=' num2str(round(rec_q.PSNR*100)/100)]);
set(f2,'position',[727   568   456   328]);

f3=figure(3); a3 = axes('parent',f3); imshow(reshape(uint8(abs(f-frec)),sizeim(1),sizeim(2)));
title(a3,'reconstruction error');
set(f3,'position',[1183  568  466   321]);

fprintf('\t\t\t\t\t\t\t\t Done.\n\n');

fprintf('Image reconstruction quality:\n')
display(rec_q);

