% ggplot(A,locations,mksize,textflag,cbflag,f,Clim) plots the graph in a figure window.
% - A is the symmetric adjacency matrix. If A is boolean, edges will be drawn
% in black. If not, colours will be used for the different weights.
% - locations is the N by 2 matrix of the [x y] positions of the N nodes 
% - mksize is the size of the nodes in the representation. 
% - Clim is the limits of the colorbar for the signal f.
% - textflag is a flag for wether or not we print the node numbers
% - cbflag is a flag for wether or not we print the colorbars
% If mksize not detailed: default is automatically estimated
% - f is a vector defined on the nodes
% If f is not detailed, default is a column of ones.
% If needed, the colorbar for the edges is plotted on the right of the
% figure and the colorbar for the nodes is plotted on the bottom of the
% figure.
% If Clim is not detailed: default is min and max of f
% If textflag is not detailed: default is 1
%
% 
% This file is part of the CoSub toolbox (Connected Subgraph Filterbanks 
% For Graph Signals Toolbox)
% Copyright (C) 2015, Nicolas Tremblay. 
%
% The CoSub toolbox is free software: you can redistribute 
% it and/or modify it under the terms of the GNU General 
% Public License as published by the Free Software 
% Foundation, either version 3 of the License, or (at your 
% option) any later version.
%
% The CoSub toolbox is distributed in the hope 
% that it will be useful, but WITHOUT ANY WARRANTY; 
% without even the implied warranty of MERCHANTABILITY 
% or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
% Public License for more details.
%
% You should have received a copy of the GNU General Public 
% License along with the MSCD_Wav toolbox. 
% If not, see <http://www.gnu.org/licenses/>.

function CSFB_ggplot(A,locations,mksize,textflag,cbflag,varargin)

% gray !
% map=gray(72);

% jet!
map=jet(72);


if ~issymmetric(A)
    error('ggplot: matrix A should be symmetric');
end

[I,J]=find(triu(A,1)~=0); % find the edges
val=A(triu(A,1)~=0); % find the weights associated to these edges

% verifications of inputs:
if isempty(mksize)
    mksize=max([2,ceil(min([10*max(sqrt((locations(I,1)-locations(J,1)).^2+(locations(I,2)-locations(J,2)).^2)), 10]))]);
end
if isempty(textflag)
    textflag=1;
end

if nargin==5
    f=ones(length(A),1);
    Clim=[min(f),max(f)];
elseif nargin==6
    f=varargin{1};
    Clim=[min(f),max(f)];
elseif nargin==7
    f=varargin{1};
    Clim=varargin{2};
end

if nargin<5 || nargin>7
	error('ggplot: problem with input');
end 
    
linwid=max([round((mksize)/4),3]); %width of edges

if min(f)==max(f)
    g=[0,0,0]; % black is default for nodes
    %g=[255,0,0];
else
    g=map;g(65:72,:)=[];
end

ech_n=linspace(Clim(1),Clim(2),64);

if length(A)==1
    scatter(locations(:,1),locations(:,2),mksize^2,'k','filled'); % plot the nodes
    return;
end

if min(val)==max(val)
    j=[0,0,0]; % black is default for edges
else
    j=map;j(65:72,:)=[];
end
ech=linspace(min(val),max(val),64);

[m,ind]=min(abs(repmat(ech,length(val),1)-repmat(val, 1, length(ech))),[],2);
ColOrd=j(ind,:);
COinit=get(gca,'ColorOrder');
set(0,'DefaultAxesColorOrder',ColOrd);
plot([locations(I,1), locations(J,1)]',[locations(I,2), locations(J,2)]','-','linewidth',linwid);

hold on;

[m,ind]=min(abs(repmat(ech_n,length(f),1)-repmat(f,1, length(ech_n))),[],2);
ColOrd=g(ind,:);
ColOrd(find(isnan(f)),:)=repmat([0,0,0],sum(isnan(f)),1);
scatter(locations(:,1),locations(:,2),mksize^2,ColOrd,'filled'); % plot the nodes

if textflag
    text(locations(:,1),locations(:,2),num2str([1:length(locations)]')            ,...
                'FontUnits'           , 'points'                      ,...
                'HitTest'             , 'off'                         ,...
                'FontWeight'          , 'normal'                        ,...
                'Margin'              , 0.01                          ,...
                'HorizontalAlignment' , 'center'                      ,... %center
                'BackGroundColor'     , 'none'                        ,...
                'Selected'            , 'off'                         ,...
                'VerticalAlignment'   , 'middle'                      ,... %middle
                'LineStyle'           , 'none'                        ,...
                'FontSize'            , round(2*mksize/3)                  ,...
                'Color'               , 'r'             );  % color of text: default is red
end

if cbflag
    if min(val)~=max(val)
        colormap(j);
        h=colorbar('location','SouthOutside');
        caxis([min(val) max(val)]);
        set(h,'fontsize',30); cbfreeze(h);
    end
    if min(f)~=max(f)
        %axis1=get(gca,'position');
        colormap(g);
        h=colorbar('location','EastOutside','fontsize',20);
        ticklab=cellfun(@str2num,h.TickLabels);
        ticklab=(ticklab-min(ticklab));
        ticklab=ticklab./max(ticklab);
        ticklab=ticklab.*(Clim(2)-Clim(1));
        ticklab=ticklab+Clim(1);
        ticklab=round(ticklab*10)/10;
        h.TickLabels=ticklab;
        %set(gca,'position',[axis1(1)/4,axis1(2:end)]);
    end
    
    
   
end
set(gca,'xtick',[])
set(gca,'ytick',[])

set(0,'DefaultAxesColorOrder','remove');
set(0,'DefaultAxesColorOrder',COinit);
end