%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   "Copyright (c) 2009 The University of Southern California"
%   All rights reserved.
%
%   Permission to use, copy, modify, and distribute this software and its
%   documentation for any purpose, without fee, and without written
%   agreement is hereby granted, provided that the above copyright notice,
%   the following two paragraphs and the author appear in all copies of
%   this software.
%
%   NO REPRESENTATIONS ARE MADE ABOUT THE SUITABILITY OF THE SOFTWARE
%   FOR ANY	PURPOSE. IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED
%   WARRANTY.
%
%   Neither the software developers, the Compression Research Group,
%   or USC, shall be liable for any damages suffered from using this
%   software.
%
%   Author: Sunil K Narang
%   Director: Prof. Antonio Ortega
%   Compression Research Group, University of Southern California
%   http://biron.usc.edu/wiki/index.php?title=CompressionGroup
%   Contact: kumarsun@usc.edu
%
%   Date last modified:	02/01/2012 Sunil K. Narang
%
%   Description:
%   This file generates QMF filter-banks on the Minnesota traffic graph
%   example given in the paper:
%% "S. K. Narang and Antonio Ortega, "Perfect Reconstruction Two-Channel
%%  Wavelet Filter-Banks For Graph Structured Data",
%  IEEE TSP also avaliable as Tech. Rep. arXiv:1106.3693v3, Dec 2011
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [] = Biorth_filterbank_demo2()
clear all
clc
close all
disp('***********Welcome to Demo 2 of Graph-QMF*********')
disp('In this demo we implement a 2-dim separable graph-QMF filterbank on Minnesota traffic graph')
addpath Graph_Generators/
addpath Graph_Colorers/
addpath Graph_kernels/
addpath Bpt_Decomp_Algos/
addpath sgwt_toolbox/
addpath toolbox/
addpath Datasets/

%% Parameters
% max_level = 3; % number of decomposition levels
filter_type = 'approximate'; % polynomial approximation of Meyer filters
norm_type='asym';

%% Section 1: Graph Formulation

% Minnesota traffic graph
[A xy] = Minnesota_traffic_graph(); % Minnesota Traffic Graph
% [A xy] = random_bipartite_graph(); % random bipartite graphs G(n,m,p)
A(348:349,:)=[];A(:,348:349)=[];
xy(348:349,:)=[];


%% Section 2: Bipartite subgraph decomposition

% Graph Coloring: F is the output coloring vector
%%%%%%%%%%%%%
% F = BSC(A); % Backtracking Sequential Coloring Algorithm: Exact but slow
% F = LF(A);   % Greedy Largest Degree First Coloring: Fast but inaccurate
% F = DSATUR(A); % Greedy Coloring based on improved heuristic: Moderate speed and accuracy
load min_coloring %  Preloaded Coloring for Minnesota graph
%%%%%%%%%%%%%%
F(348:349)=[];

% generate the downsampling functions
N = length(A);
[beta bptG beta_dist Colorednodes]= harary_decomp(A,F);
theta = size(beta,2);
Fmax = size(beta_dist,1);

%% Section 3: Graph Signal
load min_graph_signal % Preloaded graph signal for Minnesota traffic
f(348:349)=[];
% graph
if ~exist('f','var') % or
    D = diag(sum(A,2));
    d = sum(A,2);
    d(d == 0) = 1;
    d_inv = d.^(-0.5);
    D_inv = diag(d_inv);
    Ln = D_inv*A*D_inv;
    Ln = 0.5*(Ln + Ln');
    [U Lam] = eigs(Ln);
    f = U(:,4);
    f = f - min(f);
    f = im2bw(f,graythresh(f));
    f = double(f);
    f(f==0) = -1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Section 4: Filterbank implementation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Compute Normalized Laplacian Matrices for Each Bpt graph
disp('Computing normalized Laplacian matrices for each subgraph...');
switch norm_type
    case 'sym'
        Ln_bpt = zeros(N,N,theta);
        for i = 1:theta
            d1 = sum(bptG(:,:,i),2);
            d1(d1 == 0) = 1; % for isolated nodes
            d1_inv = d1.^(-0.5);
            D1_inv = diag(d1_inv);
            An = D1_inv*bptG(:,:,i)*D1_inv;
            An = 0.5*(An + An');
            Ln_bpt(:,:,i) = eye(N) - An;
            Ln_bpt(:,:,i) = (Ln_bpt(:,:,i) + (Ln_bpt(:,:,i))')./2;
        end
    case 'asym'
        Ln_bpt = zeros(N,N,theta);
        for i = 1:theta
            d1 = sum(bptG(:,:,i),2);
            d1(d1 == 0) = 1; % for isolated nodes
            d1_inv = d1.^(-1);
            D1_inv = diag(d1_inv);
            An = D1_inv*(0.5*(bptG(:,:,i) + bptG(:,:,i)'));
            Ln_bpt(:,:,i) = eye(size(An,1)) - An;
        end
        Ln_bptsym = zeros(N,N,theta);
        for i = 1:theta
            d1 = sum(bptG(:,:,i),2);
            d1(d1 == 0) = 1; % for isolated nodes
            d1_inv = d1.^(-0.5);
            D1_inv = diag(d1_inv);
            An = D1_inv*bptG(:,:,i)*D1_inv;
            An = 0.5*(An + An');
            Ln_bptsym(:,:,i) = eye(N) - An;
            Ln_bptsym(:,:,i) = (Ln_bptsym(:,:,i) + (Ln_bptsym(:,:,i))')./2;
        end
    otherwise
        disp('Unknown normalization option')
        return;
end

% design a low-pass kernel
% design biorthogonal kernels
Nd = 5;
Nr = 5;
% S = sprintf('%s%d%s ' , ' Computing a ',filterlen,' ^th order approximation of Meyer kernel');
% disp(S)
[lo_d,hi_d] = biorth_kernel_new(Nd,Nr); % Nd zeros for lowpass Nr zeros for highpass
% [hi_d,lo_d] = biorth_kernel(Nd,Nr);
filterlen_hi = length(roots(hi_d));
filterlen_lo = length(roots(lo_d));
g0 = @(x)(polyval(lo_d,x));
g1 = @(x)(polyval(hi_d,x));
h0 = @(x)(polyval(hi_d,2 - x));
h1 = @(x)(polyval(lo_d,2 - x));

arange = [0 2];
c_d{1}=sgwt_cheby_coeff(h0,filterlen_lo,filterlen_lo+1,arange);
c_d{2}=sgwt_cheby_coeff(h1,filterlen_hi,filterlen_hi+1,arange);
c_r{1}=sgwt_cheby_coeff(g0,filterlen_hi,filterlen_hi+1,arange);
c_r{2}=sgwt_cheby_coeff(g1,filterlen_lo,filterlen_lo+1,arange);

% Compute Filterbank Output at each channel
disp('Computing wavelet transform coefficients ...')
switch filter_type
    case 'approximate'
        f_w = zeros(N,Fmax);
        Channel_Name = cell(Fmax,1);
        for i = 1:Fmax
            if ~isempty(Colorednodes{i})
                tempf_w = f;
                for j = 1: theta
                    if beta_dist(i,j) == 1
                        tempf_w = sgwt_cheby_op(tempf_w,Ln_bpt(:,:,j),c_d{1},arange);
                        Channel_Name{i} = strcat(Channel_Name{i},'L');
                    else
                        tempf_w = sgwt_cheby_op(tempf_w, Ln_bpt(:,:,j),c_d{2},arange);
                        Channel_Name{i} = strcat(Channel_Name{i},'H');
                    end
                end
                f_w(Colorednodes{i},i) = tempf_w(Colorednodes{i});
            end
        end
    case 'exact'
        switch norm_type
            case 'asym'
                for i=1:size(Ln_bpt,3)
                    [U(:,:,i),D]=eig(full(Ln_bptsym(:,:,i)));
                    Lam(:,i)=diag(D);
                    U(:,:,i)=D1_inv*U(:,:,i);
                end
                f_w = zeros(N,Fmax);
                Channel_Name = cell(Fmax,1);
                for i = 1:Fmax
                    if ~isempty(Colorednodes{i})
                        tempf_w = f;
                        for j = 1: theta
                            if beta_dist(i,j) == 1
                                tempf_w = U(:,:,j)*diag(h0(Lam(:,j)))*inv(U(:,:,j))*tempf_w;
                                %tempf_w = sgwt_cheby_op(tempf_w,Ln_bpt(:,:,j),c_d{1},arange);
                                Channel_Name{i} = strcat(Channel_Name{i},'L');
                            else
                                tempf_w = U(:,:,j)*diag(h1(Lam(:,j)))*inv(U(:,:,j))*tempf_w;
%                                 tempf_w = sgwt_cheby_op(tempf_w, Ln_bpt(:,:,j),c_d{2},arange);
                                Channel_Name{i} = strcat(Channel_Name{i},'H');
                            end
                        end
                        f_w(Colorednodes{i},i) = tempf_w(Colorednodes{i});
                    end
                end
            case 'sym'
                for i=1:size(Ln_bpt,3)
                    [U(:,:,i),D]=eig(full(Ln_bpt(:,:,i)));
                    Lam(:,i)=diag(D);
                end
                f_w = zeros(N,Fmax);
                Channel_Name = cell(Fmax,1);
                for i = 1:Fmax
                    if ~isempty(Colorednodes{i})
                        tempf_w = f;
                        for j = 1: theta
                            if beta_dist(i,j) == 1
                                tempf_w = U(:,:,j)*diag(h0(Lam(:,j)))*U(:,:,j)'*tempf_w;
%                                 tempf_w = sgwt_cheby_op(tempf_w,Ln_bpt(:,:,j),c_d{1},arange);
                                Channel_Name{i} = strcat(Channel_Name{i},'L');
                            else
                                tempf_w = U(:,:,j)*diag(h1(Lam(:,j)))*U(:,:,j)'*tempf_w;
%                                 tempf_w = sgwt_cheby_op(tempf_w, Ln_bpt(:,:,j),c_d{2},arange);
                                Channel_Name{i} = strcat(Channel_Name{i},'H');
                            end
                        end
                        f_w(Colorednodes{i},i) = tempf_w(Colorednodes{i});
                    end
                end
            otherwise
                disp('Unknown normalization option')
                return;
        end
end

% 
% wav_coeffs = sum(f_w,2);
% channel_info(Fmax).name =[];
% for i = 1:Fmax
%     channel_info(i).name = Channel_Name{i};
%     channel_info(i).nodes = Colorednodes{i};
% end
% channel_info(3).name = 'HL';

f_w = find_kbest({f_w}, 0.01, 1, Colorednodes'); % threshold for nCoeffs best wavelet coefficients
f_w=cell2mat(f_w);

%% Section 5: Reconstruction
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reconstruct Signals in Original Graph Domain by using Coefficients
% Compute Filterbank Output at each channel
switch filter_type
    case 'exact'
        disp('Reconstructed signals using single channel coefficients ...')
        switch norm_type
            case 'asym'
                f_hat = zeros(N,Fmax);
                % Channel_Name = cell(Fmax,1);
                for i = 1:Fmax
                    tempf_hat = f_w(:,i);
                    for j = theta:-1: 1
                        if beta_dist(i,j) == 1
                            tempf_hat = U(:,:,j)*diag(g0(Lam(:,j)))*inv(U(:,:,j))*tempf_hat;
                        else
                            tempf_hat = U(:,:,j)*diag(g1(Lam(:,j)))*inv(U(:,:,j))*tempf_hat;
                        end
                    end
                    f_hat(:,i) = tempf_hat;
                end
            case 'sym'
                f_hat = zeros(N,Fmax);
                % Channel_Name = cell(Fmax,1);
                for i = 1:Fmax
                    tempf_hat = f_w(:,i);
                    for j = theta:-1: 1
                        if beta_dist(i,j) == 1
                            tempf_hat = U(:,:,j)*diag(g0(Lam(:,j)))*U(:,:,j)'*tempf_hat;
                        else
                            tempf_hat = U(:,:,j)*diag(g1(Lam(:,j)))*U(:,:,j)'*tempf_hat;
                        end
                    end
                    f_hat(:,i) = tempf_hat;
                end
            otherwise
                disp('Unknown normalization option')
                return;
        end
        
        
    case 'approximate'
        
        disp('Reconstructed signals using single channel coefficients ...')
        f_hat = zeros(N,Fmax);
        % Channel_Name = cell(Fmax,1);
        for i = 1:Fmax
            tempf_hat = f_w(:,i);
            for j = theta:-1: 1
                if beta_dist(i,j) == 1
                    tempf_hat = sgwt_cheby_op(tempf_hat,Ln_bpt(:,:,j),c_r{1},arange);
                else
                    tempf_hat = sgwt_cheby_op(tempf_hat, Ln_bpt(:,:,j),c_r{2},arange);
                end
            end
            f_hat(:,i) = tempf_hat;
        end
        
    otherwise
end

% %% Section 6: Summary
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% uLam = 0:1/N:2;
% if strcmp(filter_type,'exact')
%     g1 = g(uLam);
% else
%     h0 = sgwt_cheby_eval(uLam,c_d{1},arange);
%     h1 = sgwt_cheby_eval(uLam,c_d{2},arange);
%     g0 = sgwt_cheby_eval(uLam,c_r{1},arange);
%     g1 = sgwt_cheby_eval(uLam,c_r{2},arange);
% end
% % MSE_kernel = mean((g1-g(uLam)).^2);
% Err = 1 - 0.5*(h1.*g1 + h0.*g0);
% Err = max(abs(Err));
% S = sprintf('%s%f','The max Error in kernel-approximation = ' ,Err);
% disp(S);
MSE = norm(f - sum(f_hat,2))^2;
SNR = 10*log10(norm(f)^2/MSE);
S = sprintf('%s%f','The MSE between overall reconstructed graph signal and original signal = ' ,MSE/N);
disp(S);
S = sprintf('%s%f','The SNR of reconstruction = ' ,SNR);
disp(S);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Section 7: Plots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Bipartite Decomposition
% if ~exist('xy','var')
%     [x y] = draw_dot(A);
%     xy(:,1) = x'; xy(:,2) = y';
% end
% scrsz = get(0,'ScreenSize');
% height = scrsz(4)/5;
% width = scrsz(3)/5;
%
% figure,
% gplot(A,xy,'r.-')
% title('Original Graph')
% set(gcf,'Position',[30,30,width,height]);
% set(gca,'Xtick',[]);
% set(gca,'Ytick',[]);
%
% for i = 1:theta
%     figure,
%     S1 = find(beta(:,i) == 1);
%     S2 = find(beta(:,i) == -1);
%     plot(xy(S1,1),xy(S1,2),'bs');
%     hold on
%     plot(xy(S2,1),xy(S2,2),'ro');
%     gplot(bptG(:,:,i),xy,'r-');
%     S = sprintf('%s%d','bipartite subgraph # ',i);
%     xlabel('set S_2 : red circles');
%     ylabel('set S_1 : blue squares');
%     title(S)
%     set(gcf,'Position',[30+i*width,30,width,height]);
%     set(gca,'Xtick',[]);
%     set(gca,'Ytick',[]);
% end


% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Scatter Plots


% wavelet coeffs of individual channels
% % w1 = f_w(Colorednodes{1},1);
% % w2 = f_w(Colorednodes{2},2);
% % w3 = f_w(Colorednodes{3},3);
% % w4 = f_w(Colorednodes{4},4);
% %
% % % use same scale for high-pass coefficients
% % w_d = [w2(:); w4(:)];
% % w2 = abs(w2)/norm(w_d);
% % w3 = abs(w3)/norm(w_d);
% % w4 = abs(w4)/norm(w_d);

% axis([-97 -91 44 48])
% use same scale for original signal, low-pass wavelet coeffs and its low-pass reconsuction
% Data = [f(:);f_hat(:,1)];
% clim = [min(Data) max(Data)];
% f = (f - clim(1))/(clim(2)-clim(1));
% f = 2*f -1;
% f_hat(:,1) = (f_hat(:,1) - clim(1))/(clim(2)-clim(1));
% f_hat(:,1) = 2*f_hat(:,1) -1;
% w_a = (w1 - clim(1))/(clim(2)-clim(1));
% w_a = 2*w_a -1;
% f_hat_d = abs(f_hat(:,2:end));
% f_hat_d = f_hat_d /norm(f_hat_d(:));
% % % % %
% % % % % % set(gcf,'Position',[30,30+2*height+10,height,height]);
% % % % % figure,
% % % % % ti = get(gca,'TightInset');
% % % % % set(gca,'Position',[ti(1) ti(2) 1-ti(3)-ti(1) 1-ti(4)-ti(2)]);
% % % % % show_wavelet(f,xy(:,1),xy(:,2));
% % % % % set(gcf,'Position',[30,(30+2*height),width,height]);
% % % % % title('Input Signal')
% % % % % axis auto
% % % % %
% % % % % % wavelet coeffs
% % % % % figure,
% % % % % h = subplot(2,2,1);
% % % % % set(h, 'pos',[0, 0.5 ,0.4,0.4]);
% % % % % show_wavelet(w_a,xy(Colorednodes{1},1),xy(Colorednodes{1},2));
% % % % % S = sprintf('%s%s%s', 'Wavelet coeffs of ',Channel_Name{1},' channel');
% % % % % title(S)
% % % % % axis auto
% % % % %
% % % % % h = subplot(2,2,2);
% % % % % set(h, 'pos',[0.5, 0.5 ,0.4,0.4]);
% % % % % % set(gcf,'Position',[30+2*height,30+ height,height,height]);
% % % % % show_wavelet(w2,xy(Colorednodes{2},1),xy(Colorednodes{2},2));
% % % % % S = sprintf('%s%s', Channel_Name{2},' channel');
% % % % % title(S)
% % % % % axis auto
% % % % %
% % % % % h = subplot(2,2,4);
% % % % % set(h, 'pos',[0.5, 0 ,0.4,0.4]);
% % % % % show_wavelet(w4,xy(Colorednodes{4},1),xy(Colorednodes{4},2));
% % % % % S = sprintf('%s%s', Channel_Name{4},' channel');
% % % % % title(S)
% % % % % set(gcf,'Position',[30+width,(30+2*height),width,height]);
% % % % % axis auto

% reconstrcuted signals
figure,
% h = subplot(2,2,1);
% set(h, 'pos',[0, 0.5 ,0.4, 0.4]);
show_wavelet(sum(f_hat,2),xy(:,1),xy(:,2));
title('Reconstruction')
axis auto

figure,
% h = subplot(2,2,1);
% set(h, 'pos',[0, 0.5 ,0.4, 0.4]);
show_wavelet(f,xy(:,1),xy(:,2));
title('Original signal')
axis auto
% % %
% % % h = subplot(2,2,2);
% % % set(h, 'pos',[0.5, 0.5 ,0.4,0.4]);
% % % %     set(gcf,'Position',[30+(i+2)*height,30,height,height]);
% % % show_wavelet(f_hat_d(:,1),xy(:,1),xy(:,2));
% % % S = sprintf('%s%s', Channel_Name{2},' channel');
% % % title(S)
% % % axis auto
% % %
% % % h = subplot(2,2,3);
% % % set(h, 'pos',[0, 0 ,0.4,0.4]);
% % % %     set(gcf,'Position',[30+(i+2)*height,30,height,height]);
% % % show_wavelet(f_hat_d(:,2),xy(:,1),xy(:,2));
% % % S = sprintf('%s%s', 'HL',' channel');
% % % title(S)
% % % axis auto
% % %
% % % h = subplot(2,2,4);
% % % set(h, 'pos',[0.5, 0 ,0.4,0.4]);
% % % %     set(gcf,'Position',[30+(i+2)*height,30,height,height]);
% % % show_wavelet(f_hat_d(:,3),xy(:,1),xy(:,2));
% % % S = sprintf('%s%s',Channel_Name{4},' channel');
% % % title(S)
% % % set(gcf,'Position',[30+2.2*width,(30+2*height),2*width,2*height]);
% % % axis auto

% figure,
% set(gcf,'Position',[30+(i+1)*height,30,height,height]);
% show_wavelet(f_hat_d(:,2),xy(:,1),xy(:,2));
% axis auto

% for i = 1:Fmax
%     figure,
% %     set(gca,'ydir','reverse')
%    axis([-97 -91 44 48])
%     set(gcf,'Position',[30+i*height,30+2*height+10,height,height]);
%     mM = [min(f_hat(:,i)),max(f_hat(:,i))];
%     diffX = f_hat(:,i);
%     X1 = (diffX - mM(1))/(mM(2) - mM(1));
%     hold on
%     for indx = 1:length(C)
%         if all(C{indx}~=1)   % If at least one of the indices is 1,
%             % then it is an open region and we can't
%             % patch that.
%             patch(V(C{indx},1),V(C{indx},2),X1(indx)); % use color i.
%         end
%     end
%     axis([-97 -91 44 48])
%     colormap(gray)
%     S = sprintf('%s%s%s', 'Reconstructed signal from ',Channel_Name{i},' channel');
%     xlabel('Voronoi Tesselation Plot');
%     title(S)
% end

% % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % % Voronoi Tesselation Plots
% [V,C] = voronoin(xy);
% figure,
% % voronoi(xy(:,1),xy(:,2));
% % set(gca,'ydir','reverse')
% axis([-97 -91 44 48])
% title('Input Signal')
% % set(gcf,'Position',[30,30+2*height+10,height,height]);
% set(gcf,'Position',[30+(i+1)*height,30,height,height]);
% mM = [min(f),max(f)];
% diffX = f;
% X1 = (diffX - mM(1))/(mM(2) - mM(1));
% hold on
% for indx = 1:length(C)
%     if all(C{indx}~=1)   % If at least one of the indices is 1,
%         % then it is an open region and we can't
%         % patch that.
%         patch(V(C{indx},1),V(C{indx},2),X1(indx)); % use color i.
%     end
% end
% axis([-97 -91 44 48])
% colormap(gray)
%
% for i = 1:Fmax
%     figure,
% %     set(gca,'ydir','reverse')
%    axis([-97 -91 44 48])
%     set(gcf,'Position',[30+i*height,30+2*height+10,height,height]);
%     mM = [min(f_hat(:,i)),max(f_hat(:,i))];
%     diffX = f_hat(:,i);
%     X1 = (diffX - mM(1))/(mM(2) - mM(1));
%     hold on
%     for indx = 1:length(C)
%         if all(C{indx}~=1)   % If at least one of the indices is 1,
%             % then it is an open region and we can't
%             % patch that.
%             patch(V(C{indx},1),V(C{indx},2),X1(indx)); % use color i.
%         end
%     end
%     axis([-97 -91 44 48])
%     colormap(gray)
%     S = sprintf('%s%s%s', 'Reconstructed signal from ',Channel_Name{i},' channel');
%     xlabel('Voronoi Tesselation Plot');
%     title(S)
% end

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Delaunay triangulation plots
%
% minf_hat = min(f_hat(:));
% maxf_hat = max(f_hat(:));
% minf_w = min(f_w(:));
% maxf_w = max(f_w(:));
% minf = min([minf_hat minf_w]);
% maxf = max([maxf_hat maxf_w]);
% figure,
% gridDelaunay = delaunay(xy(:,1),xy(:,2));
% trimesh(gridDelaunay,xy(:,1),xy(:,2),f)
% view(0,90)
% colormap('cool')
% v = axis;
% caxis([minf maxf])
% % axis([-98 -88 43 50])
% colorbar
% set(gcf,'Position',[30,30+3*height+20,height,height]);
% title(' Graph-signal')
% xlabel('Delaunay triangulation plot')
% axis(v)
% % f_w = abs(f_w);
% % maxf = max(f_w(:));
% % minf = min(f_w(:));
% % f_w = (f_w - minf)/(maxf - minf);
% warning off
% % for i = 1:Fmax
% %     if length(Colorednodes{i}) >2 % need atleast 3 points for triangulation
% %         figure,
% %         gridDelaunay = delaunay(xy(Colorednodes{i},1),xy(Colorednodes{i},2));
% %         trimesh(gridDelaunay,xy(Colorednodes{i},1),xy(Colorednodes{i},2),f_w(Colorednodes{i},i))
% %         view(0,90)
% %         colormap('cool')
% %         set(gcf,'Position',[30+i*height,30+3*height+20,height,height]);
% %         S = sprintf('%s%s%s',Channel_Name{i},' channel ', 'wavelet coefficients');
% %         xlabel('Delaunay triangulation plot');
% %         title(S)
% %         caxis([minf maxf])
% %         colorbar
% %         axis(v);
% %     end
% % end
%
% gridDelaunay = delaunay(xy(:,1),xy(:,2));
%
% for i = 1:Fmax
%     figure,
%     trimesh(gridDelaunay,xy(:,1),xy(:,2),f_hat(:,i))
%     view(0,90)
%     colormap('cool')
%     set(gcf,'Position',[30+i*height,30+3*height+20,height,height]);
%     S = sprintf('%s%s%s', 'Reconstructed signal from ',Channel_Name{i},' channel');
%     xlabel('Delaunay triangulation plot');
%     title(S)
%     caxis([minf maxf])
%     colorbar
%     axis(v);
% end

