%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Code for Figures 8 and 9: pyramid transform of a piecewise-smooth signal
% on the bunny graph

close all;
clear all;
rand('seed',41)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate graph
graph_param.large=1;
G=gsp_create_graph('bunny',graph_param);
gsp_plot_graph(G);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate a multiresolution of graphs
num_levels=3;
multiresolution_param.compute_full_eigen=0;
multiresolution_param.sparsify=1;
multiresolution_param.sparsify_epsilon=.1;
[Gs,subsampled_vertex_indices]=gsp_graph_multiresolution(G,num_levels,multiresolution_param);

gsp_plot_graph(Gs)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate a piecewise smooth function (two polynomials of the coordinates) 
% with a discontinuity 
[smallest_eigenvectors,smallest_eigenvalues]=eigs(Gs{1}.L,2,'SM');
poly1=G.coords(:,1).^2+G.coords(:,2)-2*G.coords(:,3);
gsp_plot_graph_signal(G,poly1);
poly2=G.coords(:,1)-G.coords(:,2)+3*G.coords(:,3).^2-5;
gsp_plot_graph_signal(G,poly2);

signal=zeros(G.N,1);
signal(smallest_eigenvectors(:,1)>=0)=poly1(smallest_eigenvectors(:,1)>=0);
signal(smallest_eigenvectors(:,1)<0)=poly2(smallest_eigenvectors(:,1)<0);

% Plot the partition
gsp_plot_graph_signal(G,sign(smallest_eigenvectors(:,1)));

% Plot the signal
cmin=min(0,1.01*min(signal));
cmax=max(0,1.01*max(signal));
plot_param.climits=[cmin,cmax];
plot_param.cp=[-6.5361,-20.7675,-8.3173];
plot_param.pt_size=400;
gsp_plot_graph_signal(G,signal,plot_param);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct Filters
analysis_param.h_filters=cell(num_levels,1);
for i=1:num_levels
    analysis_param.h_filters{i}=@(x) 5./(5+x);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Analyze signal
analysis_param.use_exact=multiresolution_param.compute_full_eigen;
analysis_param.cheb_order=40;
analysis_param.regularize_epsilon=.05;
[coarse_approximations,prediction_errors]=gsp_pyramid_analysis(signal,Gs,subsampled_vertex_indices,num_levels,analysis_param);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute redundancy
redundancy_factor=(length(vertcat(prediction_errors{:}))+length(coarse_approximations{num_levels+1}))/Gs{1}.N

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot coefficients

% Plot coarse approximations
for i=1:num_levels+1
    gsp_plot_graph_signal(Gs{i},coarse_approximations{i},plot_param);
    titleStr=sprintf('$x^{(%d)}$',i-1);
    title(titleStr,'Interpreter','latex','FontSize',24);
end

% Plot prediction errors
max_abs_coeff=max(abs(vertcat(prediction_errors{:})));
plot_param.climits=[-max_abs_coeff,max_abs_coeff];
for i=1:num_levels
    gsp_plot_graph_signal(Gs{i},prediction_errors{i},plot_param);
    titleStr=sprintf('$y^{(%d)}$',i-1);
    title(titleStr,'Interpreter','latex','FontSize',24);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Synthesize the original signal from its coefficients (direct method)
synthesis_param.cheb_order=40;
synthesis_param.regularize_epsilon=.05;
reconstruction_noiseless_direct=gsp_pyramid_synthesis(coarse_approximations{num_levels+1},prediction_errors,Gs,subsampled_vertex_indices,synthesis_param);
reconstruction_errorI_noiseless_direct=norm(reconstruction_noiseless_direct-signal,Inf)
reconstruction_error2_noiseless_direct=norm(reconstruction_noiseless_direct-signal,2)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Synthesize the original signal from thresholded coefficients (direct
% method)
all_coeff=[coarse_approximations{num_levels+1};prediction_errors{1};prediction_errors{2};prediction_errors{3}];
all_coeff_sorted=sort(abs(all_coeff),'descend');
thresh=all_coeff_sorted(floor(Gs{1}.N/3)+1);
thresh_coarse_approximations=coarse_approximations;
thresh_prediction_errors=prediction_errors;
thresh_coarse_approximations{num_levels+1}(abs(coarse_approximations{num_levels+1})<thresh)=0;
for i=1:num_levels
    thresh_prediction_errors{i}(abs(prediction_errors{i})<thresh)=0;
end
reconstruction_lossy_direct=gsp_pyramid_synthesis(thresh_coarse_approximations{num_levels+1},thresh_prediction_errors,Gs,subsampled_vertex_indices,synthesis_param);
reconstruction_errorI_lossy_direct=norm(reconstruction_lossy_direct-signal,Inf)
reconstruction_error2_lossy_direct=norm(reconstruction_lossy_direct-signal,2)
average_error_lossy_direct=reconstruction_error2_lossy_direct/norm(signal,2)
figure;
plot(all_coeff_sorted,'LineWidth',4);
gsp_plot_graph_signal(Gs{1},reconstruction_lossy_direct,plot_param);


% The code for the least squares synthesis method is below, but takes
% longer to run (~15 minutes)

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Synthesize the original signal from its coefficients (least squares method)
% synthesis_param.least_squares=1;
% synthesis_param.h_filters=analysis_param.h_filters;
% reconstruction_noiseless_ls=gsp_pyramid_synthesis(coarse_approximations{num_levels+1},prediction_errors,Gs,subsampled_vertex_indices,synthesis_param);
% reconstruction_errorI_noiseless_ls=norm(reconstruction_noiseless_ls-signal,Inf)
% reconstruction_error2_noiseless_ls=norm(reconstruction_noiseless_ls-signal,2)
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Synthesize the original signal from thresholded coefficients (least
% % squares method)
% reconstruction_lossy_ls=gsp_pyramid_synthesis(thresh_coarse_approximations{num_levels+1},thresh_prediction_errors,Gs,subsampled_vertex_indices,synthesis_param);
% reconstruction_errorI_lossy_ls=norm(reconstruction_lossy_ls-signal,Inf)
% reconstruction_error2_lossy_ls=norm(reconstruction_lossy_ls-signal,2)
% average_error_lossy_ls=reconstruction_error2_lossy_ls/norm(signal,2)
% gsp_plot_graph_signal(Gs{1},reconstruction_lossy_ls,plot_param);


