%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Code for Figure 7: pyramid transform of a piecewise-constant signal
% on the Minnesota graph

close all;
clear all; 
set(0,'DefaultFigureWindowStyle','docked');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up the graph 
G=gsp_create_graph('minnesota');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate a multiresolution of graphs
num_levels=3;
multiresolution_param.compute_full_eigen=1;
multiresolution_param.sparsify=1;
multiresolution_param.sparsify_epsilon=.2;
[Gs,subsampled_vertex_indices]=gsp_graph_multiresolution(G,num_levels,multiresolution_param);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate a piecewise constant function with a discontinuity 
signal=2*(Gs{1}.U(:,2)>=0)-1;
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct Filters
analysis_param.h_filters=cell(num_levels,1);
for i=1:num_levels
    analysis_param.h_filters{i}=@(x) .5./(.5+x);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Analyze signal
analysis_param.use_exact=multiresolution_param.compute_full_eigen;
[coarse_approximations,prediction_errors]=gsp_pyramid_analysis(signal,Gs,subsampled_vertex_indices,num_levels,analysis_param);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute redundancy
redundancy_factor=(length(vertcat(prediction_errors{:}))+length(coarse_approximations{num_levels+1}))/Gs{1}.N

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot coefficients
plot_param.pt_size=400;

% Plot coarse approximations
max_abs_coeff=max(abs(vertcat(coarse_approximations{:})));
plot_param.climits=[-max_abs_coeff,max_abs_coeff];
for i=1:num_levels+1
    gsp_plot_graph_signal(Gs{i},coarse_approximations{i},plot_param);
    titleStr=sprintf('$x^{(%d)}$',i-1);
    title(titleStr,'Interpreter','latex','FontSize',24);
end

% Plot prediction errors
max_abs_coeff=max(abs(vertcat(prediction_errors{:})));
plot_param.climits=[-max_abs_coeff,max_abs_coeff];
for i=1:num_levels
    gsp_plot_graph_signal(Gs{i},prediction_errors{i},plot_param);
    titleStr=sprintf('$y^{(%d)}$',i-1);
    title(titleStr,'Interpreter','latex','FontSize',24);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Synthesize the original signal from its coefficients (direct method)
reconstruction_noiseless_direct=gsp_pyramid_synthesis(coarse_approximations{num_levels+1},prediction_errors,Gs,subsampled_vertex_indices);
reconstruction_error_noiseless_direct=norm(reconstruction_noiseless_direct-signal,Inf)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Synthesize the original signal from thresholded coefficients (direct
% method)
all_coeff=[coarse_approximations{num_levels+1};prediction_errors{1};prediction_errors{2};prediction_errors{3}];
all_coeff_sorted=sort(abs(all_coeff),'descend');
thresh=all_coeff_sorted(floor(Gs{1}.N/3)+1);
thresh_coarse_approximations=coarse_approximations;
thresh_prediction_errors=prediction_errors;
thresh_coarse_approximations{num_levels+1}(abs(coarse_approximations{num_levels+1})<thresh)=0;
for i=1:num_levels
    thresh_prediction_errors{i}(abs(prediction_errors{i})<thresh)=0;
end
reconstruction_lossy_direct=gsp_pyramid_synthesis(thresh_coarse_approximations{num_levels+1},thresh_prediction_errors,Gs,subsampled_vertex_indices);
reconstruction_errorI_lossy_direct=norm(reconstruction_lossy_direct-signal,Inf)
reconstruction_error2_lossy_direct=norm(reconstruction_lossy_direct-signal,2)
gsp_plot_graph_signal(Gs{1},reconstruction_lossy_direct);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Synthesize the original signal from its coefficients (least squares method)
synthesis_param.least_squares=1;
synthesis_param.h_filters=analysis_param.h_filters;
reconstruction_noiseless_ls=gsp_pyramid_synthesis(coarse_approximations{num_levels+1},prediction_errors,Gs,subsampled_vertex_indices,synthesis_param);
reconstruction_error_noiseless_ls=norm(reconstruction_noiseless_ls-signal,Inf)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Synthesize the original signal from its coefficients (least squares method, with Landweber)
synthesis_param.use_landweber=0;
reconstruction_noiseless_ls_lw=gsp_pyramid_synthesis(coarse_approximations{num_levels+1},prediction_errors,Gs,subsampled_vertex_indices,synthesis_param);
reconstruction_errorI_noiseless_ls=norm(reconstruction_noiseless_ls_lw-signal,Inf)
reconstruction_error2_noiseless_ls=norm(reconstruction_noiseless_ls_lw-signal,2)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Synthesize the original signal from thresholded coefficients (least
% squares method)
reconstruction_lossy_ls=gsp_pyramid_synthesis(thresh_coarse_approximations{num_levels+1},thresh_prediction_errors,Gs,subsampled_vertex_indices,synthesis_param);
reconstruction_errorI_lossy_ls=norm(reconstruction_lossy_ls-signal,Inf)
reconstruction_error2_lossy_ls=norm(reconstruction_lossy_ls-signal,2)
gsp_plot_graph_signal(Gs{1},reconstruction_lossy_ls);
