function [ G ] = gsp_create_sphere( BL , param)
%GSP_CREATE_SPHERE  Initialize a graph on an equiangular sampling of the
%sphere
%   Usage:   G=gsp_create_sphere(BL,param);
%
%   Input parameters:
%         BL                     : Bandlimit.
%         param                  : Optional variable containing additional parameters.
%   Output parameters:
%         G                      : Graph structure.
%   Additional input parameters:
%         param.weight_method    : Choices include  'inverse' (default),'unweighted', and 'gaussian'.
%         param.tau              : Parameter for Gaussian kernel edge weights if that method is used (default=1).
%
%   'gsp_create_sphere(BL,param)' initializes a graph structure containing
%   an equiangular sampling of the sphere at the bandlimit BL. There are 
%   2*BL points around each ring, BL-1 rings, and 2 poles for a total of
%   2BL(BL-1)+2 vertices. The edge weights are either all set to 1
%   ('unweighted'), or based on the geodesic distances between vertices.
%
%   See also:  
%
%   Demos:  
%
%   Requires: SSHT Toolbox from http://www.jasonmcewen.org/codes.html
% 
%   References: 
%

G.BL=BL;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct sampling positions
[theta, phi, ~, ~, nphi] = ssht_sampling(BL, 'Grid', true, 'Method', 'MWSS');

% Remove poles
theta = theta(2:end-1,:);
phi = phi(2:end-1,:);

% Vectorize
theta = theta(:);
phi = phi(:);
G.N = length(theta)+2;

% Compute Cartesian coordinates
G.coords=zeros(G.N,3);
x=sin(theta).*cos(phi);
y=sin(theta).*sin(phi);
z=cos(theta);
G.coords(2:G.N-1,:)= [x, y, z];
G.coords(1,:)=[0; 0; 1]; % North pole
G.coords(G.N,:)=[0; 0; -1]; % South pole

G.coord_limits=[-1,-1,-1;1,1,1];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate adjacency matrix
nring=2*BL;
nlev=BL-1;

% Enumerate the nonzero entries of the adjacency matrix

% MAKE THESE CALLS MORE EFFICIENT

% Add vertical edges
vert_inds=zeros((nlev-1)*nring,1);
k=0;
for i=1:nlev-1
    for j=0:nring-1
        k=k+1;
        vert_inds(k)=j*nlev+i+1;
    end
end
ver_row_locs=[vert_inds;vert_inds+1];
ver_col_locs=[vert_inds+1;vert_inds];

% Add horizontal edges
hor_start_inds=zeros(nlev*nring,1);
hor_end_inds=zeros(nlev*nring,1);
k=0;
for i=1:nlev
    for j=0:nring-2
        k=k+1;
        hor_start_inds(k)=j*nlev+i+1;
        hor_end_inds(k)=(j+1)*nlev+i+1;
    end
    k=k+1;
    hor_start_inds(k)=i+1;
    hor_end_inds(k)=(nring-1)*nlev+i+1;
end
hor_row_locs=[hor_start_inds;hor_end_inds];
hor_col_locs=[hor_end_inds; hor_start_inds];

% Connect North Pole
np_neighbors=2+(0:2*nlev:(nring-1)*nlev)';
np_row_locs=[ones(BL,1);np_neighbors];
np_col_locs=[np_neighbors;ones(BL,1)];

% Connect South Pole
sp_neighbors=nlev+1+(0:2*nlev:(nring-1)*nlev)';
sp_row_locs=[G.N*ones(BL,1);sp_neighbors];
sp_col_locs=[sp_neighbors;G.N*ones(BL,1)];

% Generate matrix
row_locs=[ver_row_locs;hor_row_locs;np_row_locs;sp_row_locs];
col_locs=[ver_col_locs;hor_col_locs;np_col_locs;sp_col_locs];
A=sparse(row_locs,col_locs,ones(size(row_locs)),G.N,G.N);

% Set the weights as a function of the geodesic distances
G.dist_geodesic = acos(G.coords*G.coords');
G.W=sparse(G.N,G.N);

if ~isfield(param,'weight_method')
    weight_method='inverse';
else
    weight_method=param.weight_method;
end

switch lower(weight_method)
    case 'unweighted'
        G.W=A;
    case 'inverse'
        G.W(A>0) = 1./G.dist_geodesic(A>0);
    case 'gaussian'
        if ~isfield(param,'tau')
            tau=1;
        else
            tau=param.tau;
        end
        G.W(A>0) = exp(-G.dist_geodesic(A>0).^2/(2*tau^2));
    otherwise
        error('Unknown weight method');
end

end

