function [f_out]=gsp_filter(f_in,G,filter,varargin)
%GSP_FILTER Graph spectral filtering
%   Usage:  filtered_signal=gsp_filter(signal,G,filter);
%           filtered_signal=gsp_filter(signal,G,filter,param);
%
%   Input parameters:
%         f_in             : Graph signal to filter.
%         G                : Graph structure.
%         filter           : Spectral graph filter.
%   Output parameters:
%         f_out            : Filtered graph signal.
%   Additional parameters:
%         param.use_exact  : To use exact graph spectral filtering instead of the Chebyshev approximation.
%         param.cheb_order : Degree of the Chebyshev approximation (default=30).
%
%   'gsp_filter(f_in,G,filter)' performs graph spectral filtering:
%
%   .. f_out = \hat{h}(L) f_in
%
%   .. math:: f_{out}=\hat{h}({\cal L}) f_{in}
%
%   where the matrix function $\hat{h}({\cal L})$ is defined as
%
%   See also:  
%
%   Demos:  
% 
%   References: 

%   AUTHOR : David I Shuman.
%   TESTING: 
%   REFERENCE:
  
% Read input parameters
if nargin>3
    param=varargin{1};
else
    param=0;
end

if ~isfield(param,'use_exact');
    use_exact=( isfield(G,'U') && isfield(G,'E') );
else
    use_exact=param.use_exact;
end

if use_exact 
    if ( ~isfield(G,'E') || ~isfield(G,'U') )
        G=gsp_full_eigen(G);
    end
    f_out=G.U*diag(filter(G.E))*G.U'*f_in;

else % use Chebyshev approximation method

    if ~isfield(param,'cheb_order');
        cheb_order=30;
    else
        cheb_order=param.cheb_order;
    end
    
    if ~isfield(G,'lmax');
        G.lmax=sgwt_rough_lmax(G.L);
    end

    cheb_coeffs=sgwt_cheby_coeff(filter,cheb_order,cheb_order+1,[0,G.lmax]);
    f_out=sgwt_cheby_op(f_in,G.L,cheb_coeffs,[0,G.lmax]);

end
