function [Gs,subsampled_vertex_indices]=gsp_graph_multiresolution(G,num_levels,varargin)
%GSP_GRAPH_MULTIRESOLUTION  Compute a multiresolution of graphs
%   Usage:  [Gs,subsampled_vertex_indices]=gsp_graph_multiresolution(G,num_levels);
%           [Gs,subsampled_vertex_indices]=gsp_graph_multiresolution(G,num_levels,param);
%
%   Input parameters:
%         G                          : Graph structure.
%         num_levels                 : Number of times to downsample and coarsen the graph.
%   Output parameters:
%         Gs                         : Cell array, with each element containing a graph structure represent a reduced graph.
%         subsampled_vertex_indices  : Indices of the vertices of the previous graph that are kept for the subsequent graph.
%   Additional parameters:
%         param.reduction_method     : The graph reduction method (default='Kron_sparsified')
%         param.compute_full_eigen   : To also compute the graph Laplacian eigenvalues for every graph in the sequence
%         
%   'gsp_graph_multiresolution(G,num_levels)' computes a multiresolution of 
%   graph by repeatedly downsampling and performing a graph reduction. The
%   default downsampling method is the largest eigenvector method based on 
%   the polarity of the components of the eigenvector associated with the 
%   largest graph Laplacian eigenvalue. The default graph reduction method
%   is Kron reduction followed by a graph sparsification step.
%
%   See also:  
%
%   Demos:  
% 
%   References: 
%

%   AUTHOR : David I Shuman.
%   TESTING: 
%   REFERENCE:

if nargin>2
    param=varargin{1};
else
    param=0;
end

if ~isfield(param,'reduction_method')
    reduction_method='kron_sparsify'; 
else
    reduction_method=param.reduction_method;
end

if ~isfield(param,'sparsify')
    sparsify=1; 
else
    sparsify=param.sparsify;
end

if ~isfield(param,'sparsify_epsilon')
    sparsify_epsilon=min(10/sqrt(G.N),.3);
else
    sparsify_epsilon=param.sparsify_epsilon;
end

if ~isfield(param,'compute_full_eigen')
    compute_full_eigen=0; 
else
    compute_full_eigen=param.compute_full_eigen;
end

graph_param.lap_type=G.lap_type;

Gs=cell(num_levels+1,1);
Gs{1}=G;
if compute_full_eigen
    Gs{1}=gsp_full_eigen(Gs{1});
end
subsampled_vertex_indices=cell(num_levels,1);

for lev=1:num_levels
    
    if Gs{lev}.N==1
        break;
    end
    
    % Downsample by keeping the nonnegative components of the largest eigenvector
    if compute_full_eigen
        largest_eigenvector = Gs{lev}.U(:,Gs{lev}.N);
    else
        [largest_eigenvector,largest_eigenvalue]=eigs(Gs{lev}.L,1);
    end
    nonnegative_logicals=(largest_eigenvector >= 0);
    if sum(nonnegative_logicals) == 0
        error('Too many pyramid levels. Try fewer.');
    end
    keep_inds=find(nonnegative_logicals==1);
    subsampled_vertex_indices{lev}=keep_inds;
       
    % Create new weighted adjacency matrix via graph reduction
    switch reduction_method
        case 'kron'
            % Kron reduction
            Gs{lev+1}.L=gsp_kron_reduce(Gs{lev}.L,keep_inds);
            
        case 'kron_sparsify'
            % Kron reduction
            Gs{lev+1}.L=gsp_kron_reduce(Gs{lev}.L,keep_inds);
            % Spectral sparsification
            if sparsify
                N=size(Gs{lev+1}.L,1);
                sparsify_epsilon=max(sparsify_epsilon,2/sqrt(N));
                [Gs{lev+1}.L,~,~] = gsp_graph_sparsify(Gs{lev+1}.L,sparsify_epsilon);
            end
            
        otherwise
            error('Unknown graph reduction method');
    end
    

    new_W=diag(diag(Gs{lev+1}.L))-Gs{lev+1}.L;
    
    % Store new graph
    graph_param.W=new_W;
    Gs{lev+1}=gsp_create_graph('custom',graph_param); 
%     Gs{lev+1}.coords=Gs{lev}.coords(keep_inds,:);
%     Gs{lev+1}.coord_limits=G.coord_limits;
%     Gs{lev+1}.edge_width=G.edge_width;
%     Gs{lev+1}.vertex_size=G.vertex_size;
    if compute_full_eigen
        Gs{lev+1}=gsp_full_eigen(Gs{lev+1});
    else
        Gs{lev}.lmax=largest_eigenvalue;
    end
    
end


end



  

